#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests the PlantUML magics """

import unittest
from contextlib import contextmanager
from classroom_extensions.plantuml import PlantUmlMagics
import classroom_extensions.plantuml as plantuml_ext
from .base import BaseTestCase

_AZURE_DIAGRAM = """
    @startuml
    !include <azure/AzureCommon>
    !include <azure/Analytics/AzureEventHub>
    !include <azure/Analytics/AzureStreamAnalyticsJob>
    !include <azure/Databases/AzureCosmosDb>

    left to right direction

    agent "Device Simulator" as devices #fff

    AzureEventHub(fareDataEventHub, "Fare Data", "PK: Medallion HackLicense VendorId; 3 TUs")
    AzureEventHub(tripDataEventHub, "Trip Data", "PK: Medallion HackLicense VendorId; 3 TUs")
    AzureStreamAnalyticsJob(streamAnalytics, "Stream Processing", "6 SUs")
    AzureCosmosDb(outputCosmosDb, "Output Database", "1,000 RUs")

    devices --> fareDataEventHub
    devices --> tripDataEventHub
    fareDataEventHub --> streamAnalytics
    tripDataEventHub --> streamAnalytics
    streamAnalytics --> outputCosmosDb
    @enduml
"""

_SIMPLE_DIAGRAM = """
    actor Foo1
    boundary Foo2
    control Foo3
    entity Foo4
    database Foo5
    Foo1 -> Foo2 : To boundary
    Foo1 -> Foo3 : To control
    Foo1 -> Foo4 : To entity
    Foo1 -> Foo5 : To database
"""

_SIMPLE_JSON = """
{
    "status": "OK",
    "code": 200,
    "total": 2,
    "data": [
        {
            "title": "Harum cumque placeat id.",
            "description": "Qui autem tenetur ut aut.",
            "url": "https://placekitten.com/300/500"
        },
        {
            "title": "Incidunt neque at enim fuga.",
            "description": "Harum libero quo dolorum aut vel.",
            "url": "https://placekitten.com/300/400"
        }
    ]
}
"""

_TEST_PLANTUML_URL = "http://localhost:8080/plantuml/"
_DEFAULT_PLANTUML_URL = "http://plantuml.com/plantuml/"
_DEFAULT_FORMAT = "svg"


class TestPlantUML(BaseTestCase):
    """Testcase for the PlantUML extension"""

    @contextmanager
    def _config(self, server, out_format):
        magics = PlantUmlMagics(shell=self.ipython)
        magics.plantuml_config(f"--server={server} --format={out_format}")
        yield magics
        magics.plantuml_config(
            f"--server={_DEFAULT_PLANTUML_URL} --format={_DEFAULT_FORMAT}"
        )

    def test_config(self):
        """Tests creating a config file"""
        print("Testing PlantUML config...")
        with self._config(server=_TEST_PLANTUML_URL, out_format="png") as magic:
            self.assertEqual(_TEST_PLANTUML_URL, magic.plantweb_config["server"])
            self.assertEqual("png", magic.plantweb_config.get("format"))
        with self._config(server=_TEST_PLANTUML_URL, out_format="svg") as magic:
            self.assertEqual(_TEST_PLANTUML_URL, magic.plantweb_config["server"])
            self.assertEqual("svg", magic.plantweb_config.get("format"))
        magic = PlantUmlMagics(self.ipython)
        output = self.capture_output(magic.plantuml_config, "--format=png")
        self.assertRegex(output, "Use --server=address to")

    def test_render_svg(self):
        """Tests rendering an svg"""
        print("Testing PlantUML rendering SVG...")
        with self._config(server=_DEFAULT_PLANTUML_URL, out_format="svg") as magic:
            try:
                magic.plantuml(cell=_SIMPLE_DIAGRAM)
            except Exception as exception:
                self.fail(f"Error: {exception}")

    def test_render_json(self):
        """Tests rendering an svg"""
        print("Testing PlantUML rendering JSON...")
        with self._config(server=_DEFAULT_PLANTUML_URL, out_format="png") as magic:
            try:
                magic.json(cell=_SIMPLE_JSON)
            except Exception as exception:
                self.fail(f"Error: {exception}")

        tmp_file = "/tmp/temp_json.json"
        with open(tmp_file, "w", encoding="UTF-8") as json_file:
            json_file.write(_SIMPLE_JSON)

        with self._config(server=_DEFAULT_PLANTUML_URL, out_format="png") as magic:
            try:
                magic.json(line=f"--json={tmp_file}", cell=None)
            except Exception as exception:
                self.fail(f"Error: {exception}")

    def test_render_png(self):
        """Tests rendering a png"""
        print("Testing PlantUML rendering PNG...")
        magic = PlantUmlMagics(self.ipython)
        magic.plantuml_config(f"--server={_DEFAULT_PLANTUML_URL}")
        try:
            magic.plantuml(line="--format=png", cell=_AZURE_DIAGRAM)
        except Exception as exception:
            self.fail(f"Error: {exception}")

    def test_load_extension(self):
        """Tests loading and unloading the extension"""
        print("Testing loading/unloading extension...")
        self.ipython.extension_manager.load_extension("classroom_extensions.plantuml")
        second_load = self.ipython.extension_manager.load_extension(
            "classroom_extensions.plantuml"
        )
        self.assertEqual(second_load, "already loaded")

    def test_incorrect_loading(self):
        """Tests incorrectly loading the extension."""
        expected = "IPython shell not available.\n"
        output = self.capture_output(plantuml_ext.load_ipython_extension, None)
        self.assertEqual(output, expected)
        output = self.capture_output(plantuml_ext.unload_ipython_extension, None)
        self.assertEqual(output, expected)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests the MariaDB magics """

import unittest
import json
from os import path
import classroom_extensions.mariadb as mariadb_ext
from .base import BaseTestCase

MARIADB_USER = "testuser"
MARIADB_PASSWORD = "testpassword"


class TestMariaDB(BaseTestCase):
    """Testcase for the MariaDB extension"""

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()

        # Create the MariaDB kernel config file
        cls._create_mariadb_config()

    def setUp(self):
        # Custom path to MariaDB kernel config
        self.ipython.run_line_magic("env", "JUPYTER_CONFIG_DIR /tmp/")

        # Load the mariadb extension
        self.ipython.extension_manager.load_extension("classroom_extensions.mariadb")

    def tearDown(self):
        self.ipython.extension_manager.unload_extension("classroom_extensions.mariadb")

    def test_show_databases(self):
        """Tests the execution of SQL command"""
        print("Testing SHOW DATABASES command.")
        self.ipython.run_cell_magic("sql", line="", cell="SHOW DATABASES;")
        pattern = r"<TABLE BORDER=1><TR><TH>Database</TH></TR><TR><TD>(.*?)</TD></TR>.+</TABLE>"
        self.assertRegex(
            text=str(self.publisher.display_output.pop()), expected_regex=pattern
        )

    def test_bad_sql(self):
        """Tests the execution of a bad SQL command"""
        print("Testing bad SQL command.")
        output = self.capture_output(
            self.ipython.run_cell_magic, "sql", line="", cell="SELEC * FROM H;"
        )
        self.assertRegex(output, "Error")

    @classmethod
    def _create_mariadb_config(cls):
        """
        Create the MariaDB kernel config file required by the MariaDB extension.
        The file contains information on how to access the MariaDB server
        """

        config_path = path.join("/tmp", "mariadb_config.json")
        client_conf = {
            "user": f"{MARIADB_USER}",
            "host": "localhost",
            "port": 3306,
            "password": f"{MARIADB_PASSWORD}",
            "start_server": "False",
            "client_bin": "/usr/bin/mariadb",
            "socket": "/var/run/mysqld/mysqld.sock",
        }
        with open(config_path, "w", encoding="utf-8") as config_file:
            config_file.write(json.dumps(client_conf, indent=4))
            config_file.flush()

    def test_incorrect_loading(self):
        """Tests incorrectly loading the extension."""
        expected = "IPython shell not available.\n"
        output = self.capture_output(mariadb_ext.load_ipython_extension, None)
        self.assertEqual(output, expected)
        output = self.capture_output(mariadb_ext.unload_ipython_extension, None)
        self.assertEqual(output, expected)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests the JavaScript magics """

import unittest
from os import path

from IPython.utils import io

import classroom_extensions.web as node_ext
from classroom_extensions.web import JavascriptWithConsole, HTMLWithConsole
from .base import BaseTestCase


class TestNodeJs(BaseTestCase):
    """Testcase for the NodeJs extension"""

    def setUp(self) -> None:
        # Loads the extension
        self.ipython.extension_manager.load_extension("classroom_extensions.web")

    def tearDown(self) -> None:
        self.ipython.extension_manager.unload_extension("classroom_extensions.web")

    def test_node_script(self):
        """Tests executing server-side JavaScript"""
        print("Test executing Node.js script.")
        cell_output: str
        console_content = "------"
        with io.capture_output() as captured:
            self.ipython.run_cell_magic(
                "javascript",
                line="--target=node --filename=/tmp/test.js",
                cell=f"console.log('{console_content}');\n",
            )
            cell_output = captured.stdout
        self.assertEqual(cell_output.strip(), console_content)

    def test_save_on_disk(self):
        """Tests saving script to disk"""
        print("Test on saving a script on disk.")
        tmp_file = "/tmp/test_disk.js"
        self.ipython.run_cell_magic(
            "javascript",
            line=f"--target=disk --filename={tmp_file}",
            cell="console.log('------');\n",
        )
        self.assertEqual(path.exists(tmp_file), True)
        try:
            self.ipython.run_cell_magic(
                "javascript", line="--target=disk", cell="console.log(' ');\n"
            )
        except ValueError:
            pass

    # def test_node_server(self):
    #     """Tests the creation of a Node.js server"""
    #     print("Testing executing Node.js server...")
    #     cell_output: str
    #     cell_content = """
    #         const http = require('http')
    #
    #         const hostname = 'localhost'
    #         const port = process.env.NODE_PORT || 3000
    #
    #         const server = http.createServer((req, res) => {
    #             res.statusCode = 200
    #             res.setHeader('Content-Type', 'text/plain')
    #             res.end('Hello world!')
    #         })
    #
    #         server.listen(port, hostname, () => {
    #             console.log(`Server listening at http://${hostname}:${port}/`)
    #         })
    #     """
    #     with io.capture_output() as captured:
    #         self.ipython.run_cell_magic(
    #             "javascript",
    #             line="--target=node --filename=/tmp/server.js --port=3000",
    #             cell=f"{cell_content}",
    #         )
    #         cell_output = captured.stdout
    #     self.assertRegex(cell_output.strip(), r"(Killing|Server)")

    def test_javascript(self):
        """Tests normal JavaScript code"""
        print("Testing JavaScript with console...")

        expected_dir = {
            "text/plain": f"<{JavascriptWithConsole.__module__}."
            f"{JavascriptWithConsole.__qualname__} object>"
        }
        cell_content = "console.log('----');"
        self.ipython.run_cell_magic("javascript", line="", cell=f"{cell_content}")
        self.assertEqual(expected_dir, self.publisher.display_output.pop())

    def test_html_javascript(self):
        """Test HTML with JavaScript"""
        print("Testing HTML with JavaScript")
        expected_dir = {
            "text/plain": f"<{HTMLWithConsole.__module__}."
            f"{HTMLWithConsole.__qualname__} object>"
        }
        cell_content = "console.log('----');"
        self.ipython.run_cell_magic("html", line="--console", cell=f"{cell_content}")
        self.assertEqual(expected_dir, self.publisher.display_output.pop())

    def test_html_console(self):
        """Tests the HTML with console."""

        html_code = """
            <div class="container">
            <h1>An H1 Title</h1>
            <h2>An H2 Title</h2>
            <p>A paragraph with some text.</p>
            </div>
        """

        html = HTMLWithConsole(data=html_code, console=True)
        self.assertRegex(html._repr_html_(), "console_elems")

    def test_incorrect_loading(self):
        """Tests incorrectly loading the extension."""
        expected = "IPython shell not available.\n"
        output = self.capture_output(node_ext.load_ipython_extension, None)
        self.assertEqual(output, expected)
        output = self.capture_output(node_ext.unload_ipython_extension, None)
        self.assertEqual(output, expected)

    def test_http_server(self):
        """Tests start/stop of HTTP server"""
        self.ipython.run_line_magic(
            "http_server",
            line="--action=start --bind=0.0.0.0 --port=8000 --directory=/tmp",
        )
        self.ipython.run_line_magic("http_server", line="--action=stop --port=8000")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
An extension to install MongoDB and MongoDB Shell (mongosh).

Note: This extension assumes that you are working in Google Colab
running Ubuntu 22.04.
"""
from os import path
import os
import glob
import time
from argparse import ArgumentParser
from IPython.core.getipython import get_ipython
from IPython.core.magic import magics_class, line_magic, Magics
from .util import exec_cmd, get_os_release, is_colab

__all__ = ["load_ipython_extension", "unload_ipython_extension", "MongoDBInstaller"]

_START_DB_TIMEOUT = 5  # Timeout for starting MongoDB
_SOFTWARE_DESC = {"mongo": "MongoDB"}

_INSTALL_CMDS = {
    "mongo": [
        "apt update -y",
        "apt-get install gnupg curl",
        """curl -fsSL https://pgp.mongodb.com/server-6.0.asc | \
             gpg -o /usr/share/keyrings/mongodb-server-6.0.gpg \
             --dearmor""",
        "sudo apt-get install gnupg",
        "echo 'deb [ arch=amd64,arm64 signed-by=/usr/share/keyrings/mongodb-server-6.0.gpg ] "
        "https://repo.mongodb.org/apt/ubuntu jammy/mongodb-org/6.0 multiverse' "
        "| tee /etc/apt/sources.list.d/mongodb-org-6.0.list",
        "apt update -y",
        "apt-get install -y mongodb-org",
    ],
}

_SAMPLE_DBS_URL = "https://github.com/neelabalan/mongodb-sample-dataset.git"


@magics_class
class MongoDBInstaller(Magics):
    """
    Implements the behaviour of the magic for installing MongoDB
    and MongoDB Shell on Google Colab.
    """

    in_notebook: bool
    _arg_parser: ArgumentParser

    def __init__(self, shell):
        super().__init__(shell=shell)
        self._arg_parser = self._create_parser()
        self.in_notebook = shell.has_trait("kernel")

    @staticmethod
    def _create_parser() -> ArgumentParser:
        parser = ArgumentParser()
        parser.add_argument(
            "-s",
            "--sample_dbs",
            action="store_true",
            help="To import the sample databases",
        )
        return parser

    @staticmethod
    def _meet_requirements() -> bool:
        """
        Check if running on Colab with the right Ubuntu release

        Returns:
            True if running on Google Colab on Ubuntu 2x.xx container
        """
        return is_colab() and get_os_release().startswith("2")

    @staticmethod
    def install_software(software: str) -> None:
        """Installs a given software"""
        description = _SOFTWARE_DESC.get(software)
        commands = _INSTALL_CMDS.get(software)

        print(f"Installing {description}...")
        try:
            for cmd in commands:
                exec_cmd(cmd)
            print(f"{description} is installed.")
        except RuntimeError as runtime_error:
            print(f"Error installing {description}: {runtime_error}")

    @staticmethod
    def import_sample_datasets() -> None:
        """Clones the git repository with multiple sample datasets and imports them"""
        local_clone = "sample_dbs"
        print("Cloning git repository with the sample datasets...")
        clone_path = path.join(os.getcwd(), local_clone)
        try:
            if not path.exists(clone_path):
                exec_cmd(f"git clone {_SAMPLE_DBS_URL} {local_clone}")
            else:
                print("Skipping git clone as local repository seems to exist.")

            datasets = [
                f
                for f in os.listdir(local_clone)
                if not path.isfile(path.join(local_clone, f))
            ]
            for dataset in datasets:
                dataset_path = path.join(clone_path, dataset)
                print(f"Importing dataset {dataset}...")
                for json_file in glob.glob(f"{dataset_path}/*.json"):
                    collection = path.splitext(path.basename(json_file))[0]
                    cmd = (
                        f"mongoimport --drop --host localhost --port 27017 "
                        f"--db {dataset} --collection {collection} --file {json_file}"
                    )
                    exec_cmd(cmd)
            print("Finished importing the sample datasets.")
        except RuntimeError as runtime_error:
            print(f"Error importing sample databases: {runtime_error}")

    @staticmethod
    def _start_mongodb() -> None:
        """Starts MongoDB"""

        get_ipython().system_raw("mongod --config /etc/mongod.conf &")
        print("Waiting for a few seconds for MongoDB server to start...")
        time.sleep(_START_DB_TIMEOUT)

    @line_magic
    def install_mongodb(self, line: str):
        """Install MongoDB and MongoDB Shell"""

        if not self._meet_requirements():
            print(
                "Note: the magics for installing and configuring "
                "MongoDB may not work outside Google Colab"
            )

        args = self._arg_parser.parse_args(line.split() if line else "")
        self.install_software("mongo")
        self._start_mongodb()

        if args.sample_dbs:
            self.import_sample_datasets()


def load_ipython_extension(ipython):
    """
    Loads the ipython extension

    Args:
        ipython: (InteractiveShell) The currently active `InteractiveShell` instance.

    Returns:
        None
    """
    try:
        mongodb_installer = MongoDBInstaller(ipython)
        ipython.register_magics(mongodb_installer)
        ipython.mongodb_installer = mongodb_installer
    except (NameError, AttributeError):
        print("IPython shell not available.")


def unload_ipython_extension(ipython):
    """Does some clean up"""

    try:
        del ipython.mongodb_installer
    except (NameError, AttributeError):
        print("IPython shell not available.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests the MongoDB magics """

import unittest
import os
import uuid
from classroom_extensions.mongodb import MongoDBConfig
import classroom_extensions.mongodb as mongodb_ext
from .base import BaseTestCase

MONGO_USERNAME = "admin"
MONGO_PASSWORD = "password"
MONGO_DATABASE = "testdb"

PARAM_LINE = (
    f"--host=localhost --port=27017 "
    f"--username={MONGO_USERNAME} "
    f"--password={MONGO_PASSWORD}"
)


class TestMongoDB(BaseTestCase):
    """Testcase for the MongoDB extension"""

    config_file = None

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.config_file = f"{uuid.uuid4().hex}.json"

    def setUp(self) -> None:
        # Load the extension
        self.ipython.extension_manager.load_extension("classroom_extensions.mongodb")

        # Custom path to config files
        self.ipython.run_line_magic("env", "JUPYTER_CONFIG_DIR /tmp")

    def tearDown(self):
        self.ipython.extension_manager.unload_extension("classroom_extensions.mongodb")

    @classmethod
    def tearDownClass(cls) -> None:
        super().tearDownClass()
        os.unlink(os.path.join("/tmp", cls.config_file))

    def test_new_config(self):
        """Tests creating a new config file"""
        print("Testing new config...")
        config = MongoDBConfig(config_file=self.config_file)
        config.save()
        expected = " --host localhost --port 27017"
        self.assertEqual(expected, config.get_shell_args())

    def test_config_magic(self):
        """Tests the mongo_config line magic"""
        print("Testing mongo_config magic")
        self.ipython.run_line_magic("mongo_config", line=PARAM_LINE)

    def test_cell_magic(self):
        """Tests executing the mongo cell magic"""
        print("Testing mogo magic")
        js_code = """show dbs;
        use testdb;
        show collections;
        """
        self.capture_output(
            self.ipython.run_cell_magic, "mongo", line=PARAM_LINE, cell=js_code
        )

    def test_incorrect_loading(self):
        """Tests incorrectly loading the extension."""
        expected = "IPython shell not available.\n"
        output = self.capture_output(mongodb_ext.load_ipython_extension, None)
        self.assertEqual(output, expected)
        output = self.capture_output(mongodb_ext.unload_ipython_extension, None)
        self.assertEqual(output, expected)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Some helper functions """

import subprocess
import os
import pwd


_IN_COLAB = False
try:
    import google.colab

    _IN_COLAB = True
except ModuleNotFoundError:
    _IN_COLAB = False


def get_os_release() -> str:
    """Get Ubuntu's release number (e.g. 20.04)"""
    rls_version: str = ""
    with open("/etc/os-release", "r", encoding="utf-8") as release_file:
        for line in release_file:
            if line.startswith("VERSION_ID="):
                rls_version = line.split("=")[1].strip('" \n')
                break
    return rls_version


def exec_cmd(command: str) -> None:
    """
    Execute a command and print error, if occurs

    Args:
        command: the command to execute

    Returns:
        None
    """
    try:
        subprocess.check_output(
            f"{command} > /dev/null", shell=True, stderr=subprocess.STDOUT
        )
    except subprocess.CalledProcessError as process_error:
        print(f"Error occurred: {process_error.output.decode()}")


def is_colab() -> bool:
    """
    Check if running on Google Colab

    Returns:
        True if running on Google Colab
    """
    return _IN_COLAB


def get_user() -> str:
    """Get the username of the user the code runs under"""

    uid = os.getuid()
    return pwd.getpwuid(uid).pw_name


def is_extension() -> bool:
    """
    Check if the code has been loaded with %load_ext

    Returns:
        True if loaded as extension via %load_ext
    """
    return "__IPYTHON__" in globals()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Used to run commands on the MongoDB shell """

from argparse import Namespace
from subprocess import CalledProcessError
from os import environ, path
import json
import shutil
import sys
from IPython.core.magic import magics_class, cell_magic, line_magic
from IPython.core.magics.display import DisplayMagics
from IPython.core.magics.script import ScriptMagics
from IPython.core import magic_arguments
from IPython.utils.process import arg_split

__all__ = [
    "load_ipython_extension",
    "unload_ipython_extension",
    "MongoDBConfig",
    "MongoDBMagics",
]


def mongosh_args(func):
    """Single decorator for adding mongosh args"""
    args = [
        magic_arguments.argument(
            "--host", help="The hostname or IP address of the MongoDB server"
        ),
        magic_arguments.argument(
            "--port",
            type=int,
            help="The port number on which the MongoDB server is listening",
        ),
        magic_arguments.argument("--username", help="The username for authentication"),
        magic_arguments.argument("--password", help="The password for authentication"),
        magic_arguments.argument(
            "--authenticationDatabase", help="The database used for authentication"
        ),
        magic_arguments.argument(
            "--tls", action="store_true", help="Enable TLS/SSL for the connection"
        ),
        magic_arguments.argument(
            "--tlsCAFile", type=open, help="The path to the CA certificate file"
        ),
        magic_arguments.argument(
            "--tlsCertificateKeyFile",
            type=open,
            help="The path to the client certificate and private key file",
        ),
    ]
    for arg in args:
        func = arg(func)
    return func


class MongoDBConfig:
    """To keep the configuration on how to connect to MongoDB"""

    def __init__(self, config_file: str = "mongodb_extension.json"):
        self._config_file = config_file
        self._config = {
            "host": "localhost",
            "port": 27017,
            "username": None,
            "password": None,
            "authenticationDatabase": None,
            "tls": False,
            "tlsCAFile": None,
            "tlsCertificateKeyFile": None,
        }
        self._load_config()

    def update_config(self, config: dict = None) -> None:
        """
        Updates the configuration with the
        values in the provided dictionary

        Args:
            config: a dictionary with configuration parameters

        Returns:
            None
        """
        self._config.update(config)

    def _load_config(self) -> None:
        config_path = self._config_path()
        if path.exists(config_path):
            with open(config_path, "r", encoding="utf-8") as config_file:
                try:
                    self._config.update(json.loads(config_file.read()))
                except json.JSONDecodeError as json_error:
                    print(f"Error reading MongoDB config: {json_error}")

    def _config_path(self) -> str:
        """Gets the path to the configuration file"""
        default_dir = path.join(path.expanduser("~"), ".jupyter")
        custom_dir = environ.get("JUPYTER_CONFIG_DIR")
        if custom_dir:
            default_dir = custom_dir
        return path.join(default_dir, self._config_file)

    def save(self) -> None:
        """Saves the configuration into the config file"""
        config_path = self._config_path()
        with open(config_path, "w", encoding="utf-8") as config_file:
            config_file.write(json.dumps(self._config))

    def get_shell_args(self, args: dict = None) -> str:
        """
        Returns a list of arguments a mongosh command line

        Args:
            args: a dictionary with the mongosh command line arguments

        Returns:
            The command line for mongosh
        """
        command = ""
        arg_list = args if args else self._config
        for arg, value in arg_list.items():
            if value is not None:
                if isinstance(value, bool):
                    if value:
                        command += f" --{arg}"
                else:
                    command += f" --{arg} {value}"

        return command

    def copy(self):
        """
        Creates a shallow copy of the configuration object

        Returns:
            A shallow copy of the object
        """
        clone = MongoDBConfig(self._config_file)
        clone.update_config(self._config)
        return clone


@magics_class
class MongoDBMagics(DisplayMagics):
    """Defines the mongo shell magics"""

    _config: MongoDBConfig = MongoDBConfig()
    _script_magics: ScriptMagics

    def __init__(self, shell=None):
        super().__init__(shell=shell)
        self.log = shell.log
        self._mongosh = self._shell_path()
        self._script_magics = ScriptMagics(shell=shell)

    def _shell_path(self) -> str:
        shell_path = shutil.which("mongosh")
        if shell_path:
            return shell_path

        self.log.error("Command mongosh not found.")
        raise FileNotFoundError("Error: mongosh not found. Please install mongosh")

    @magic_arguments.magic_arguments()
    @mongosh_args
    @cell_magic("mongo")
    def mongo(self, line: str = "", cell: str = None) -> None:
        """
        Implements the %%mongo cell magic
        Args:
            line: the line after the magic, containing command line arguments to mongosh
            cell: the cell contents with MongoDB Shell commands

        Returns:
            None
        """
        # Check whether the user provided mongosh parameters with the cell
        argv = arg_split(line, posix=not sys.platform.startswith("win"))
        args = self.mongo.parser.parse_args(argv)
        config = self._config.copy()
        config.update_config(vars(args))

        # The flow here is not sophisticated. We simply leverage the
        # ScriptMagics' script that creates a process, writes the cell
        # contents to its stdin and creates stream readers that read from
        # stdout and stderr to render the execution results
        try:
            params = f"--quiet --no-raise-error {config.get_shell_args()}"
            cmd = f"{self._mongosh} {params}"
            self._script_magics.shebang(line=cmd, cell=cell.strip())
        except CalledProcessError as mongo_exception:
            print(f"Error while executing mongosh: {mongo_exception}")

    def _set_config(self, args: Namespace):
        new_config = {}
        for arg, value in vars(args).items():
            if value:
                new_config[arg] = value

        self._config.update_config(new_config)

    @magic_arguments.magic_arguments()
    @mongosh_args
    @line_magic
    def mongo_config(self, line: str = None) -> None:
        """
        Implements the %mongo_config magic

        Args:
            line: the containing the command line arguments to Mongo Shell

        Returns:
            None
        """
        argv = arg_split(line, posix=not sys.platform.startswith("win"))
        args = self.mongo_config.parser.parse_args(argv)
        self._set_config(args)
        self._config.save()


def load_ipython_extension(ipython):
    """
    Loads the ipython extension

    Args:
        ipython: (InteractiveShell) The currently active `InteractiveShell` instance.

    Returns:
        None
    """
    try:
        mongo_magics = MongoDBMagics(ipython)
        ipython.register_magics(mongo_magics)
        ipython.mongo_magics = mongo_magics
    except (NameError, AttributeError):
        print("IPython shell not available.")


def unload_ipython_extension(ipython):
    """Does some clean up"""

    try:
        del ipython.mongo_magics
    except (NameError, AttributeError):
        print("IPython shell not available.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
An extension to install MariaDB, create the config file required by the
MariaDB Jupyter to access it, and load a sample database.

Note: This extension assumes that you are working in Google Colab
running Ubuntu 22.04.
"""
import time
from os import path
import json
from argparse import ArgumentParser
from IPython.core.magic import magics_class, line_magic, Magics
from IPython.core.getipython import get_ipython
from .util import exec_cmd, get_os_release, is_colab, get_user

__all__ = ["load_ipython_extension", "unload_ipython_extension", "MariaDBInstaller"]

_SAMPLE_DB = "https://www.mariadbtutorial.com/wp-content/uploads/2019/10/nation.zip"
_START_DB_TIMEOUT = 5  # Timeout for starting MariaDB


@magics_class
class MariaDBInstaller(Magics):
    """
    Implements the behaviour of the magic for installing MariaDB on Google Colab.
    """

    in_notebook: bool
    _arg_parser: ArgumentParser
    _db_user: str
    _db_pass: str

    def __init__(self, shell):
        super().__init__(shell=shell)
        self._arg_parser = self._create_parser()
        self.in_notebook = shell.has_trait("kernel")
        self._db_user = get_user()
        self._db_pass = ""

    @staticmethod
    def _create_parser() -> ArgumentParser:
        parser = ArgumentParser()
        parser.add_argument(
            "-p",
            "--password",
            type=str,
            default=None,
            help="the password for the root user",
        )
        parser.add_argument(
            "-s",
            "--sample_db",
            action="store_true",
            help="to load the sample database",
        )
        return parser

    @staticmethod
    def _meet_requirements() -> bool:
        """
        Check if running on Colab with the right Ubuntu release

        Returns:
            True if running on Google Colab on Ubuntu 2x.xx container
        """
        return is_colab() and get_os_release().startswith("2")

    @staticmethod
    def _start_mariadb() -> None:
        """Starts MariaDB"""

        service_name = "mariadb" if get_os_release().startswith("22") else "mysql"
        get_ipython().system_raw(f"service {service_name} start &")
        print("Waiting for a few seconds for MariaDB server to start...")
        time.sleep(_START_DB_TIMEOUT)

    @line_magic
    def install_mariadb(self, line: str):
        """Install MariaDB, mariadb_kernel, sqlparse, etc"""

        if not self._meet_requirements():
            print(
                "Note: the magics for installing and configuring "
                "MariaDB may not work outside Google Colab"
            )

        args = self._arg_parser.parse_args(line.split() if line else "")
        if args.password is None:
            print("Error: you must provide a password using --password=password")
            return

        self._db_pass = args.password
        load_sample_db = args.sample_db
        print("Running apt update...")
        exec_cmd("apt update -y")
        print("Installing MariaDB...")
        exec_cmd("apt install mariadb-server libmariadb-dev libmariadb3 -y")
        print("Installing required python packages...")
        exec_cmd("pip3 install mariadb==1.0.11 mariadb_kernel==0.2.0 sqlparse==0.4.4")

        self._start_mariadb()  # First start MariaDB

        sql_stmt = (
            f"ALTER USER '{self._db_user}'@'localhost' IDENTIFIED BY '{self._db_pass}'"
        )
        exec_cmd(f'mariadb -e "{sql_stmt}"')
        exec_cmd("mkdir -p ~ /.jupyter")  # the config file must go in .jupyter
        config_path = path.join(path.expanduser("~"), ".jupyter/mariadb_config.json")
        client_conf = {
            "user": "root",
            "host": "localhost",
            "port": "3306",
            "password": self._db_pass,
            "start_server": "False",
            "client_bin": "/usr/bin/mariadb",
            "server_bin": "/usr/bin/mariadbd",
            "socket": "/run/mysqld/mysqld.sock",
        }
        with open(config_path, "w", encoding="utf-8") as config_file:
            config_file.write(json.dumps(client_conf, indent=4))

        # Load the sample database, if required
        if load_sample_db:
            self._load_sample_db()
        print("Done.")

    def _load_sample_db(self):
        """Configure a sample MariaDB database"""
        exec_cmd(f"wget {_SAMPLE_DB}")
        exec_cmd("unzip -o nation.zip")
        print("Importing nation database...")
        exec_cmd(
            f'mariadb -e "source nation.sql" --user={self._db_user} --password={self._db_pass}'
        )
        exec_cmd("rm nation.zip")


def load_ipython_extension(ipython):
    """
    Loads the ipython extension

    Args:
        ipython: (InteractiveShell) The currently active `InteractiveShell` instance.

    Returns:
        None
    """
    try:
        mariadb_installer = MariaDBInstaller(ipython)
        ipython.register_magics(mariadb_installer)
        ipython.mariadb_installer = mariadb_installer
    except (NameError, AttributeError):
        print("IPython shell not available.")


def unload_ipython_extension(ipython):
    """Does some clean up"""

    try:
        del ipython.mariadb_installer
    except (NameError, AttributeError):
        print("IPython shell not available.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Extension that uses PlantUML to draw multiple types of diagrams """

import json
import sys
from os.path import expanduser

from IPython.core import magic_arguments
from IPython.core.magic import magics_class, cell_magic, line_magic, line_cell_magic
from IPython.core.magics.display import DisplayMagics, display
from IPython.display import SVG, Image
from IPython.utils.process import arg_split
from plantweb import render, defaults

__all__ = ["load_ipython_extension", "unload_ipython_extension", "PlantUmlMagics"]

DEFAULT_PLANTWEB_CONFIG = defaults.DEFAULT_CONFIG


def plantuml_args(func):
    """Single decorator for adding plantuml args"""
    args = [
        magic_arguments.argument(
            "-s", "--server", help="Address of the PlantUML server to use"
        ),
        magic_arguments.argument(
            "-f",
            "--format",
            choices=["svg", "png"],
            default="svg",
            help="The output format to used",
        ),
    ]
    for arg in args:
        func = arg(func)
    return func


@magics_class
class PlantUmlMagics(DisplayMagics):
    """
    Implements magics for using PlantUML and enabling creating
    several types of diagrams in Jupyter notebooks
    """

    _config_path: str = expanduser("~/.plantwebrc")
    """ The default path of the config file """

    def __init__(self, shell=None):
        super().__init__(shell=shell)
        self._plantweb_config = defaults.read_defaults()

    @property
    def plantweb_config(self) -> dict:
        """Returns the plantweb configuration"""
        return self._plantweb_config

    def _save_plantuml_config(self) -> None:
        with open(self._config_path, "w", encoding="utf-8") as config_file:
            config_file.write(json.dumps(self._plantweb_config))

    @magic_arguments.magic_arguments()
    @plantuml_args
    @cell_magic("plantuml")
    def plantuml(self, line: str = "", cell: str = None):
        """Cell magic responsible for rendering the SVG/PNG diagram"""
        argv = arg_split(line, posix=not sys.platform.startswith("win"))
        args = self.plantuml.parser.parse_args(argv)
        server = args.server if args.server else self._plantweb_config["server"]

        output, out_format, _, _ = render.render(
            cell, server=server, engine="plantuml", format=args.format
        )
        if out_format == "svg":
            svg = SVG(data=output)
            display(svg)
        else:
            img = Image(data=output)
            display(img)

    @magic_arguments.magic_arguments()
    @plantuml_args
    @line_magic
    def plantuml_config(self, line=None) -> None:
        """
        Used to set the server address in case one
        wants to use its local PlatUML server
        """
        argv = arg_split(line, posix=not sys.platform.startswith("win"))
        args = self.plantuml_config.parser.parse_args(argv)

        if args.server:
            self._plantweb_config["server"] = args.server
        else:
            print(
                "Use --server=address to provide the address of a valid PlantUML server"
            )

        self._plantweb_config["format"] = args.format
        self._save_plantuml_config()

    @magic_arguments.magic_arguments()
    @plantuml_args
    @magic_arguments.argument("--json", "-j", type=str, help="Path to the file on disk")
    @line_cell_magic
    def json(self, line="", cell=None) -> None:
        """Used to create a graphical representation of a JSON file/object"""
        args = magic_arguments.parse_argstring(self.json, line)

        if args.json is not None:
            with open(args.json, "r", encoding="UTF-8") as json_file:
                cell = json_file.read()

        cell = f"""
                @startjson
                {cell}
                @endjson
                """
        command = ""
        for arg, value in vars(args).items():
            if value is not None and arg != "json":
                command += f" --{arg}={value}"

        self.plantuml(line=command, cell=cell)


def load_ipython_extension(ipython) -> None:
    """
    To unload the extension
    Args:
        ipython: the current interactive shell

    Returns:
        None
    """
    try:
        uml_magics = PlantUmlMagics(ipython)
        ipython.register_magics(uml_magics)
        ipython.plantuml_magics = uml_magics
    except (NameError, AttributeError):
        print("IPython shell not available.")


def unload_ipython_extension(ipython) -> None:
    """
    To unload the extension

    Args:
        ipython (InteractiveShell): Currently active `InteractiveShell` instance.

    Returns:
        None
    """
    try:
        del ipython.plantuml_magics
    except (NameError, AttributeError):
        print("IPython shell not available.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
The code customizes IPython's %%javascript magic by introducing line arguments
that allow for executing the JavaScript code on the server side (via Node.js).
When executed on the browser, the magic creates a section in the code cell that
mimics the browser's console. It also customizes `%%html` to enable the result
section of the cell to mimic the browser's console.
"""

from functools import partial
from typing import Any, AnyStr, Dict
from os import path, environ
from threading import Thread, Event
import io
import uuid
import shutil
import sys
import os
import subprocess
import psutil
from IPython.core.magic import magics_class, cell_magic, line_magic
from IPython.core.magics.display import DisplayMagics
from IPython.display import display, Javascript, HTML
from IPython.core import magic_arguments
from IPython.utils.process import arg_split

__all__ = [
    "load_ipython_extension",
    "unload_ipython_extension",
    "WebMagics",
    "JavascriptWithConsole",
    "HTMLWithConsole",
    "NodeProcessManager",
]

# timeout to wait for a node server process to start (in seconds)
START_SERVER_TIMEOUT = 5

# This code JavaScript will be included with the cell's code to
# redirect the output of calls to console.[log, error, warn] to the
# result section of the cell
_CELL_CONSOLE = """
function c_msg(type, o_func, ...args) {
    let p = document.createElement("p");
    p.classList.add(`console-${type}`);
    p.textContent = args.join(" ");
    document.getElementById('console-box').appendChild(p);
    o_func(...args);
}

const o_log = console.log.bind(console)
const o_error = console.error.bind(console);
const o_warn = console.warn.bind(console);

console.log = c_msg.bind(console, 'log', o_log);
console.error = c_msg.bind(console, 'error', o_error);
console.warn = c_msg.bind(console, 'warn', o_warn);

window.addEventListener("error", (event) => {
    console.error(`${event.type}: ${event.message}`);
});

var console_elems = {}
console_elems.stl = document.createElement('style');
console_elems.stl.textContent = `
:root {
    --font-log: Consolas, Monaco, 'Courier New', monospace;
}

.console-box {
    max-width: 70vw;
}

.console-error, .console-log, .console-warn {
    font-family: var(--font-log);
    white-space: nowrap;
    font-weight: 520;
    font-size: 0.9rem;
    line-height: 1.1rem;
    padding: 2px 10px;
    overflow-y: auto;
    border-bottom: 1px solid #A9A9A9;
    color: black;
    margin: 0;
}

.console-error {
    color: #8B0000;
    border-bottom-color: #FFC0CB;
    background-color: #FFE4E1;
}

.console-warn {
    color: #A0522D;
    border-bottom-color: #FFDEAD;
    background-color: #FFFACD;
}

@media (max-width: 600px) {
    .console-box {
        max-width: 95vw;
    }
}

@media (max-width: 992px) {
    .console-box {
        max-width: 90vw;
    }
}

@media (min-width: 993px) {
    .console-box {
        max-width: 85vw;
    }
}

@media (min-width: 1200px) {
    .console-box {
        max-width: 70vw;
    }
}
`;
document.head.appendChild(console_elems.stl);
console_elems.c_box = document.createElement('div');
console_elems.c_box.className = 'console-box';
console_elems.c_box.id = 'console-box';
document.getElementById('output-footer').appendChild(console_elems.c_box);
"""

_CONSOLE_TITLE = """
var console_elems = {}
console_elems.stl = document.createElement('style');
console_elems.stl.textContent = `
:root {
    --font-title: 'Lato', 'Lucida Grande', 'Lucida Sans Unicode', Tahoma, Sans-Serif;
}
.console-title {
    font-family: var(--font-title);
    font-weight: 700;
    color: black;
    font-size: 1.1rem;
    line-height: 1;
    padding: 9px 10px;
    white-space: nowrap;
    margin: 0;
}
`;
document.head.appendChild(console_elems.stl);
console_elems.h_title = document.createElement('h2');
console_elems.h_title.className = 'console-title';
console_elems.h_title.textContent = 'Console:';
document.getElementById('output-footer').appendChild(console_elems.h_title);
"""


class OutputReader(Thread):
    """
    Thread that receives a process and reads its stdout,
    printing the content to the output section of the cell.
    """

    def __init__(self, proc):
        super().__init__()
        self._stop_event = Event()
        self._proc = proc
        self.start()

    def stop(self) -> None:
        """Stops the thread execution"""
        self._stop_event.set()

    def run(self) -> None:
        """
        Runs the thread while no event for its termination is received.

        Returns: None

        """
        while not self._stop_event.is_set():
            for line in iter(self._proc.stdout.readline, ""):
                line = line.strip()
                if len(line) > 0:
                    print(line)


class NodeProcessManager:
    """Used to manage the execution of Node processes"""

    _node_cmd: str = "/usr/bin/node"
    _daemons: Dict[int, Any] = {}

    def __init__(self):
        # Try to discover full path of node command
        self._node_cmd = shutil.which("node")
        popen_kwargs = {
            "stdout": subprocess.PIPE,
            "stderr": subprocess.STDOUT,
            "encoding": "utf-8",
        }
        self.popen = partial(subprocess.Popen, **popen_kwargs)

    def execute(self, js_file: str = None, port: int = None) -> None:
        """
        Use Node.js to run the provided script. If a port is given,
        the script will be run in background without blocking the cell

        Args:
            js_file: the full path to the JavaScript file
            port: the port number for the server

        Returns:
            None
        """

        def process_wait(process, timeout=None):
            # Creates a thread to read the process stdout
            reader = OutputReader(process)
            finished = False

            try:
                # Wait until the process finishes, if it is not a server
                if timeout:
                    process.wait(timeout)
                else:
                    process.wait()
                finished = True
            except subprocess.TimeoutExpired:
                pass
            finally:
                reader.stop()  # Free the resources to avoid 100% CPU usage
            return finished

        is_daemon = False
        start_new_session = False
        server_env = environ.copy()
        if port:
            self.kill_daemon(port)  # Kill any Node process using the port
            server_env["NODE_PORT"] = str(port)
            is_daemon = True
            start_new_session = True

        work_dir = path.dirname(path.realpath(js_file))
        cmd_args = (self._node_cmd, js_file)

        try:
            proc = self.popen(
                cmd_args,
                env=server_env,
                cwd=work_dir,
                start_new_session=start_new_session,
            )
            if is_daemon:
                self.kill_daemon(port)
                # Wait for START_SERVER_TIMEOUT seconds for the server process to start and
                # read its output until the timeout expires
                process_wait(proc, START_SERVER_TIMEOUT)
                self._daemons[port] = proc
            else:
                process_wait(proc)
                proc.stdout.close()
        except subprocess.SubprocessError as sub_error:
            print(f"Error executing node script: {sub_error}")

    @staticmethod
    def _force_kill(port: int) -> None:
        """To kill a Node.js process listening on a given port"""
        for proc in psutil.process_iter(["pid", "name", "connections"]):
            try:
                for conn in proc.connections():
                    if conn.status == psutil.CONN_LISTEN and conn.laddr.port == port:
                        proc.kill()
            except (psutil.AccessDenied, psutil.NoSuchProcess):
                pass

    def kill_daemon(self, port: int) -> None:
        """
        Kills a previously started background process

        Args:
            port: the port the daemon is likely listening to

        Returns:
            None
        """
        if port in self._daemons:
            process = self._daemons[port]
            try:
                process.kill()
            except ProcessLookupError:
                print(f"No node process on port {port}, ok to continue...")
        else:
            self._force_kill(port)

    def clean_up(self) -> None:
        """Some cleanup during testing and extension unloading"""
        for proc in self._daemons.values():
            try:
                proc.terminate()
            except ProcessLookupError as process_error:
                print(f"Error: process not found {process_error}")
        self._daemons.clear()

    def __del__(self):
        self.clean_up()


class JavascriptWithConsole(Javascript):
    """
    This class extends JavaScript to intercept calls to console.log
    and make a result section of the cell.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _repr_javascript_(self):
        """Creates the full JavaScript code to be delivered to the browser"""
        return _CELL_CONSOLE + super()._repr_javascript_()


class HTMLWithConsole(HTML):
    """
    This adds a copy of the browser's console with the messages
    triggered when loading/executing the HTML/JavaScript code.
    """

    def __init__(self, data: AnyStr = None, console: bool = False):
        """
        Creates a new object representing the HTML code to be rendered.

        Args:
            data: the HTML content
            console: True if the browser console must be displayed
        """
        super().__init__(data=data)
        self.console = console

    def _repr_html_(self) -> str:
        """
        Creates the HTML content.

        Returns:
            The HTML code to be rendered.
        """
        html: str = ""
        if self.console:
            html += f"<script>{_CONSOLE_TITLE}{_CELL_CONSOLE}</script>"
        return html + super()._repr_html_()


def javascript_args(func):
    """Single decorator for adding JavaScript args"""
    args = [
        magic_arguments.argument(
            "-t",
            "--target",
            type=str,
            choices=["browser", "node", "disk"],
            default="browser",
            help="the target for script execution",
        ),
        magic_arguments.argument(
            "-f",
            "--filename",
            type=str,
            help="filename when cell contents are saved to disk",
        ),
        magic_arguments.argument(
            "-p",
            "--port",
            type=int,
            help="a port number if the cell starts a Node server process",
        ),
    ]
    for arg in args:
        func = arg(func)
    return func


def html_args(func):
    """Single decorator for adding HTML args"""
    args = [
        magic_arguments.argument(
            "-c",
            "--console",
            action="store_true",
            help="Whether to display a copy of the browser's console or not",
        )
    ]
    for arg in args:
        func = arg(func)
    return func


def http_server_args(func):
    """Single decorator for adding HTTP server args"""
    args = [
        magic_arguments.argument(
            "--action",
            "-a",
            default="start",
            choices=["start", "stop"],
            help="action to execute (default: start)",
        ),
        magic_arguments.argument(
            "--bind",
            "-b",
            default="0.0.0.0",
            help="specify alternate bind address (default: all interfaces)",
        ),
        magic_arguments.argument(
            "--directory",
            "-d",
            default=os.getcwd(),
            help="specify alternate directory (default: current directory)",
        ),
        magic_arguments.argument(
            "--port",
            "-p",
            default=8000,
            type=int,
            help="specify alternate port (default: 8000)",
        ),
    ]
    for arg in args:
        func = arg(func)
    return func


@magics_class
class WebMagics(DisplayMagics):
    """
    Implements the customizations to the %%javascript magic
    that enables JavaScript execution by Node.js
    """

    _proc_mgmt: NodeProcessManager
    _in_notebook: bool

    def __init__(self, shell):
        super().__init__(shell=shell)
        self._proc_mgmt = NodeProcessManager()
        self._in_notebook = shell.has_trait("kernel")

    @staticmethod
    def _save_script(filename: str, cell_content: str) -> str:
        """
        Creates the JavaScript file for Node to run

        Args:
            filename: the file name
            cell_content: the cell contents to save into the file

        Returns:
            Name of the file created
        """
        if not filename:
            filename = f"{uuid.uuid4().hex}.js"
        with io.open(filename, "w", encoding="utf-8") as script_file:
            script_file.write(cell_content)
        return filename

    def _run_on_node(self, js_file: str, port: int = None) -> None:
        """
        Triggers the execution on Node.js

        Args:
            js_file: path to the file to execute
            port: the port number to use, if the scripts launches a server

        Returns:
            None
        """
        self._proc_mgmt.execute(js_file, port)

    @magic_arguments.magic_arguments()
    @javascript_args
    @cell_magic
    def javascript(self, line: str = None, cell: str = None) -> None:
        """
        Method called when executing %%javascript
        Args:
            line: line arguments (e.g. --target, --filename)
            cell: the JavaScript code to execute

        Returns:
            None
        """
        argv = arg_split(line, posix=not sys.platform.startswith("win"))
        args = self.javascript.parser.parse_args(argv)

        if args.target == "node":
            if not args.filename:
                raise ValueError("--filename is required when using --target=node")
            js_file = self._save_script(args.filename, cell)
            self._run_on_node(js_file, args.port)
        elif args.target == "browser":
            display(JavascriptWithConsole(cell))
        elif args.target == "disk":
            if not args.filename:
                raise ValueError("--filename is required when using --target=disk")
            self._save_script(args.filename, cell)

    @magic_arguments.magic_arguments()
    @html_args
    @cell_magic
    def html(self, line=None, cell=None) -> None:
        argv = arg_split(line, posix=not sys.platform.startswith("win"))
        args = self.html.parser.parse_args(argv)
        html = HTMLWithConsole(cell, args.console)
        display(html)

    @magic_arguments.magic_arguments()
    @http_server_args
    @line_magic
    def http_server(self, line=None) -> None:
        """Line magic to start/stop a python Web server"""
        argv = arg_split(line, posix=not sys.platform.startswith("win"))
        args = self.http_server.parser.parse_args(argv)

        def start_web_server(address, directory, port):
            stop_web_server(port)  # Stop any server that is currently running
            print(f"Starting server listening on port {port}...")
            self.shell.system_raw(
                f"python -m http.server {port} --bind {address} --directory {directory} &"
            )

        def stop_web_server(port):
            print(f"Stopping any server listening on port {port}...")
            stop_cmd = (
                r"ps -x | grep -e '[h]ttp.server\s*"
                + str(port)
                + "' | awk '{print $1}' | xargs -I {} kill {}"
            )
            self.shell.system_raw(stop_cmd)

        if args.action == "start":
            start_web_server(args.bind, args.directory, args.port)
        else:
            stop_web_server(args.port)


def load_ipython_extension(ipython):
    """
    Loads the ipython extension

    Args:
        ipython: (InteractiveShell) The currently active `InteractiveShell` instance.

    Returns:
        None
    """
    try:
        web_magics = WebMagics(ipython)
        ipython.register_magics(web_magics)
        ipython.web_magics = web_magics
    except (NameError, AttributeError):
        print("IPython shell not available.")


def unload_ipython_extension(ipython):
    """Unloads the extension"""
    try:
        del ipython.web_magics
    except (NameError, AttributeError):
        print("IPython shell not available.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Set of IPython extensions used for teaching at ETS Montreal.
"""

__version__ = '0.0.10'

__all__ = [
    "mariadb",
    "mariadb_install",
    "web",
    "plantuml",
    "mongodb",
    "mongodb_install"
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
An extension to create the %%sql magic command using the MariaDB kernel to
execute the commands using the CLI client. Although it uses the MariaDB kernel, using it as
an extension does not limit the notebook to executing only SQL commands. This assumes you have

Note: MariaDB server and client installed, and that you have created a MariaDB kernel
configuration file before loading this extension.
"""
from IPython.core.magic import magics_class, cell_magic
from IPython.core.magics.display import DisplayMagics
from IPython.display import display, HTML
from mariadb_kernel.code_parser import CodeParser
from mariadb_kernel.mariadb_client import MariaDBClient, ServerIsDownError
from mariadb_kernel.client_config import ClientConfig

__all__ = ["load_ipython_extension", "unload_ipython_extension", "MariaDBMagics"]


@magics_class
class MariaDBMagics(DisplayMagics):
    """Implements the MariaDB magics using the database
    client provided by the MariaDB kernel"""

    _db_client: MariaDBClient
    _in_notebook: bool

    def __init__(self, shell):
        super().__init__(shell=shell)
        self._in_notebook = shell.has_trait("kernel")
        self.log = shell.log
        config = ClientConfig(self.log)
        self._db_client = MariaDBClient(self.log, config)
        self._db_client.start()

    @cell_magic
    def sql(self, line: str = "", cell: str = None) -> None:
        """
        Code to intercept the SQL code and execute it using MariaDB iPython kernel.

        Args:
            line: Not used, included to avoid error
            cell: The contents of the cell to execute

        Returns:
            None
        """
        parser: CodeParser
        try:
            parser = CodeParser(self.log, cell, ";")
        except ValueError as value_error:
            print(f"Error with SQL parser: {str(value_error)}")
            return

        result = ""
        for stmt in parser.get_sql():
            result += self._db_client.run_statement(stmt)

            if self._db_client.iserror():
                print(f"Error: {self._db_client.error_message()}")
                continue

        display_obj = HTML(result) if self._in_notebook else result
        display(display_obj)


def load_ipython_extension(ipython):
    """
    Loads the ipython extension

    Args:
        ipython: (InteractiveShell) The currently active `InteractiveShell` instance.

    Returns:
        None
    """
    try:
        mariadb_magics = MariaDBMagics(ipython)
        ipython.register_magics(mariadb_magics)
        ipython.mariadb_magics = mariadb_magics
    except ServerIsDownError as server_error:
        ipython.log.error(f"Error trying to access MariaDB Server: {server_error}")
    except (NameError, AttributeError):
        print("IPython shell not available.")


def unload_ipython_extension(ipython):
    """Does some clean up"""

    try:
        del ipython.mariadb_magics
    except (NameError, AttributeError):
        print("IPython shell not available.")
